@@ -124,25 +124,23 @@ GList **pkg_info(GList *all_pkgs)
   FILE *fd;
   
   gchar *p = NULL;
-  gchar *cmd;
-  gchar str[100], s[20], *desc;
+  gchar str[100], s[20];
+  GString *desc, *cmd;
 
   all_pkgs_copy = all_pkgs;
 
-  desc = g_malloc(2048);
-  cmd = g_malloc(2048);
 
-  strcpy(cmd, "/usr/bin/dpkg -s");
+  cmd = g_string_new("/usr/bin/dpkg -s");
 
   while (all_pkgs_copy) {
     p = all_pkgs_copy->data;
     all_pkgs_copy = all_pkgs_copy->next;
 
-    strcat(cmd, " ");
-    strcat(cmd, p);
+     g_string_append(cmd, " ");
+     g_string_append(cmd, p);
   }
 
-  if ((fd = popen(cmd, "r")) == NULL) {
+  if ((fd = popen(cmd->str, "r")) == NULL) {
     fprintf(stderr, "Can't open pipe for dpkg\n");
     return NULL;
   }
@@ -161,7 +159,7 @@ GList **pkg_info(GList *all_pkgs)
 
     
     if (strstr(str, "Description: ")) {
-      strcpy(desc, str + 12);
+      desc = g_string_new( str + 12);
       fgets(str, 100, fd);
       
       do {
@@ -169,21 +167,21 @@ GList **pkg_info(GList *all_pkgs)
 		int str_len = strlen(str);
 	      	if (!strcmp(str+(str_len-3)," .\n"))
 		     	str[str_len-2] = ' ';
-	        strcat(desc, str);
+	        g_string_append(desc, str);
 	}
       	fgets(str, 100, fd);
       } while (!feof(fd) && !strstr(str,"Package: ") );
 
-      p = strdup(desc);
+      p = desc->str;
 
       pkg_descriptions = g_list_append(pkg_descriptions, p);
+      g_string_free(desc,0);	/* free wrapper but keep string */
     }
   }
 
   fclose(fd);
 
-  g_free(cmd);
-  g_free(desc);
+  g_string_free(cmd,1);
 
   pkg_info[0] = pkg_sections;
   pkg_info[1] = pkg_descriptions;
